@@ -28,10 +28,7 @@
 # The rest of the savegame format was gleaned from 13xforever/Ilya's
 # "gvas-converter" project: https://github.com/13xforever/gvas-converter
 
-import base64
 import struct
-import random
-import binascii
 import google.protobuf
 import google.protobuf.json_format
 from . import *
@@ -58,27 +55,7 @@ class BL3Item(object):
     def __init__(self, protobuf, datawrapper):
         self.protobuf = protobuf
         self.datawrapper = datawrapper
-        self.serial_db = datawrapper.serial_db
-        self.name_db = datawrapper.name_db
-        self.parsed = False
-        self.can_parse = True
-        (self.decrypted_serial, self.orig_seed) = BL3Item._decrypt_serial(self.protobuf.item_serial_number)
-
-        # Attributes which get filled in when we parse
-        self._version = None
-        self._balance_bits = None
-        self._balance_idx = None
-        self._balance = None
-        self._balance_short = None
-        self._eng_name = None
-        self._invdata_bits = None
-        self._invdata_idx = None
-        self._invdata = None
-        self._manufacturer_bits = None
-        self._manufacturer_idx = None
-        self._manufacturer = None
-        self._level = None
-        self._remaining_data = None
+        self.serial = datalib.BL3Serial(self.protobuf.item_serial_number, datawrapper)
 
     @staticmethod
     def create(datawrapper, serial_number, pickup_order_idx, skin_path='', is_seen=True, is_favorite=False, is_trash=False):
@@ -105,289 +82,6 @@ class BL3Item(object):
                 weapon_skin_path=skin_path,
                 ), datawrapper)
 
-    @staticmethod
-    def _xor_data(data, seed):
-        """
-        Run some `data` through some XOR-based obfuscation, using the
-        specified `seed`
-        """
-
-        # If the seed is 0, we basically don't do anything (though
-        # make sure we return the same datatype as below)
-        if seed == 0:
-            return [d for d in data]
-
-        # Because our seed can be negative, we do have to do the
-        # & here, even though it might not seem to make sense to
-        # do so.
-        xor = (seed >> 5) & 0xFFFFFFFF
-        temp = []
-        for i, d in enumerate(data):
-            xor = (xor * 0x10A860C1) % 0xFFFFFFFB
-            temp.append((d ^ xor) & 0xFF)
-        return temp
-
-    @staticmethod
-    def _bogodecrypt(data, seed):
-        """
-        "Decrypts" the given item `data`, using the given `seed`.
-        """
-
-        # First run it through the xor wringer.
-        temp = BL3Item._xor_data(data, seed)
-
-        # Now rotate the data
-        steps = (seed & 0x1F) % len(data)
-        return bytearray(temp[-steps:] + temp[:-steps])
-
-    @staticmethod
-    def _bogoencrypt(data, seed):
-        """
-        "Encrypts" the given `data`, using the given `seed`
-        """
-
-        # Rotate first
-        steps = (seed & 0x1F) % len(data)
-        rotated = bytearray(data[steps:] + data[:steps])
-
-        # Then run through the xor stuff
-        return bytearray(BL3Item._xor_data(rotated, seed))
-
-    @staticmethod
-    def _decrypt_serial(serial):
-        """
-        Decrypts (really just de-obfuscates) the serial number.
-        """
-
-        # Initial byte should always be 3
-        assert(serial[0] == 3)
-
-        # Seed does need to be an unsigned int
-        orig_seed = struct.unpack('>i', serial[1:5])[0]
-
-        # Do the actual "decryption"
-        decrypted = BL3Item._bogodecrypt(serial[5:], orig_seed)
-
-        # Grab the CRC stored in the serial itself
-        orig_checksum = bytearray(decrypted[:2])
-
-        # Compute the checksum ourselves to make sure we've done
-        # everything properly
-        data_to_checksum = serial[:5] + b"\xFF\xFF" + decrypted[2:]
-        computed_crc = binascii.crc32(data_to_checksum)
-        computed_checksum = struct.pack('>H',
-                ((computed_crc >> 16) ^ computed_crc) & 0xFFFF)
-        if orig_checksum != computed_checksum:
-            raise Exception('Checksum in serial ({}) does not match computed checksum ({})'.format(
-                '0x{}'.format(''.join(f'{d:02X}' for d in orig_checksum)),
-                '0x{}'.format(''.join(f'{d:02X}' for d in computed_checksum)),
-                ))
-
-        # Return what we decrypted
-        return (decrypted[2:], orig_seed)
-
-    @staticmethod
-    def _encrypt_serial(data, seed=None):
-        """
-        Given an unencrypted `data`, return the binary serial number for
-        the item, optionally with the given `seed`.  If `seed` is not passed in,
-        a random one will be passed in.  Use a `seed` of `0` to not apply any
-        encryption/obfuscation to the data
-        """
-
-        # Pick a random seed if one wasn't given.  Taken from the BL2 CLI editor
-        if seed is None:
-            seed = random.randrange(0x100000000) - 0x80000000
-
-        # Construct our header and find the checksum
-        header = struct.pack('>Bi', 3, seed)
-        crc32 = binascii.crc32(header + b"\xFF\xFF" + data)
-        checksum = struct.pack('>H', ((crc32 >> 16) ^ crc32) & 0xFFFF)
-
-        # Return the freshly-encrypted item
-        return header + BL3Item._bogoencrypt(checksum + data, seed)
-
-    def _get_inv_db_header_part(self, category, bits):
-        """
-        Given the category name `category`, and the ArbitraryBits object `bits`,
-        containing serial number data, return a tuple containing:
-            1) The category value
-            2) The number of bits the category takes up
-            3) The numerical index of the value
-        This relies on being run during `_parse_serial`, so that `_version` is
-        populated in our object.
-        """
-        num_bits = self.serial_db.get_num_bits(category, self._version)
-        part_idx = bits.eat(num_bits)
-        part_val = self.serial_db.get_part(category, part_idx)
-        if not part_val:
-            part_val = 'unknown'
-        return (part_val, num_bits, part_idx)
-
-    def _parse_serial(self):
-        """
-        Parse our serial number, at least up to the level.  We're not going
-        to care about actual parts in here.
-        """
-
-        if not self.can_parse:
-            return
-
-        bits = datalib.ArbitraryBits(self.decrypted_serial)
-
-        # First value should always be 128, apparently
-        assert(bits.eat(8) == 128)
-
-        # Grab the serial version and check it against the max version we know about
-        self._version = bits.eat(7)
-        if self._version > self.serial_db.max_version:
-            self.can_parse = False
-            return
-
-        # Now the rest of the data we care about.
-        (self._balance,
-                self._balance_bits,
-                self._balance_idx) = self._get_inv_db_header_part('InventoryBalanceData', bits)
-        (self._invdata,
-                self._invdata_bits,
-                self._invdata_idx) = self._get_inv_db_header_part('InventoryData', bits)
-        (self._manufacturer,
-                self._manufacturer_bits,
-                self._manufacturer_idx) = self._get_inv_db_header_part('ManufacturerData', bits)
-        self._level = bits.eat(7)
-        self._remaining_data = bits.data
-
-        # At this point, if we were planning on reading parts, we'd read eat six
-        # more bits to find the number of parts, then read in that many, using a
-        # mapping to get to the correct category.  Then another four bits tells
-        # us how many anointments there are, reading from InventoryGenericPartData
-        # after that.  After anointments, I'm not totally sure what's out there.
-
-        # Parse out a "short" balance name, for convenience's sake
-        self._balance_short = self._balance.split('.')[-1]
-
-        # If we know of an English name for this balance, use it
-        self._eng_name = self.name_db.get(self._balance_short)
-
-        # Mark down that we're parsed, now.
-        self.parsed = True
-
-    def _deparse_serial(self):
-        """
-        De-parses a serial; used after we make changes to the data that gets
-        pulled out during `_parse_serial`.  At the moment, that's only going
-        to be item level changes.  Will update the serial in the protobuf as
-        well, and set the object to trigger a re-parse if anything else needs
-        to read more.
-
-        This is all pretty inefficient -- really we should just write over the
-        bits values with the new values, in-place.  But whatever, this'll do
-        for now.
-        """
-
-        if not self.can_parse:
-            return
-
-        # Construct the new item data
-        bits = datalib.ArbitraryBits()
-        bits.append_value(128, 8)
-        bits.append_value(self._version, 7)
-        bits.append_value(self._balance_idx, self._balance_bits)
-        bits.append_value(self._invdata_idx, self._invdata_bits)
-        bits.append_value(self._manufacturer_idx, self._manufacturer_bits)
-        bits.append_value(self._level, 7)
-        bits.append_data(self._remaining_data)
-        new_data = bits.get_data()
-
-        # Encode the new serial (using seed 0; unencrypted)
-        new_serial = BL3Item._encrypt_serial(new_data, 0)
-
-        # Load in the new serial (this will set `parsed` to `False`)
-        self.set_serial_number_data(new_serial)
-
-    @property
-    def balance(self):
-        """
-        Returns the balance for this item
-        """
-        if not self.parsed:
-            self._parse_serial()
-            if not self.can_parse:
-                return None
-        return self._balance
-
-    @property
-    def balance_short(self):
-        """
-        Returns the "short" balance for this item
-        """
-        if not self.parsed:
-            self._parse_serial()
-            if not self.can_parse:
-                return None
-        return self._balance_short
-
-    @property
-    def eng_name(self):
-        """
-        Returns an English name for the balance, if possible.  Will default
-        to the "short" balance for this item if not.
-        """
-        if not self.parsed:
-            self._parse_serial()
-            if not self.can_parse:
-                return None
-        if self._eng_name:
-            return self._eng_name
-        else:
-            return self._balance_short
-
-    @property
-    def level(self):
-        """
-        Returns the level of this item
-        """
-        if not self.parsed:
-            self._parse_serial()
-            if not self.can_parse:
-                return None
-        return self._level
-
-    @level.setter
-    def level(self, value):
-        """
-        Sets a new level for the item.  This would be a super inefficient way of
-        doing it if we supported doing anything other than changing level -- we're
-        rebuilding the whole serial right now and triggering a re-parse if anything
-        decides to re-read it.  That should be sufficient for our purposes here,
-        though.
-        """
-        if not self.parsed:
-            self._parse_serial()
-            if not self.can_parse:
-                return None
-
-        # Set the level and trigger a re-encode of the serial
-        self._level = value
-        self._deparse_serial()
-
-    def get_serial_number(self, orig_seed=False):
-        """
-        Returns the binary item serial number.  If `orig_seed` is `True`, the
-        serial number will use the same seed that was used in the savegame.
-        Otherwise, it will use a seed of `0`, which will then be unencrypted.
-        """
-        return BL3Item._encrypt_serial(self.decrypted_serial, 0)
-
-    def get_serial_base64(self, orig_seed=False):
-        """
-        Returns the base64-encoded item serial number.  If `orig_seed` is
-        `True`, the serial number will use the same seed that was used in the
-        savegame.  Otherwise, it will use a seed of `0`, which will then be
-        unencrypted.
-        """
-        return 'BL3({})'.format(base64.b64encode(self.get_serial_number(orig_seed)).decode('latin1'))
-
     def get_pickup_order_idx(self):
         return self.protobuf.pickup_order_index
 
@@ -396,19 +90,55 @@ class BL3Item(object):
         Overwrites this item with a new one
         """
         self.protobuf.item_serial_number = new_data
-        (self.decrypted_serial, self.orig_seed) = BL3Item._decrypt_serial(self.protobuf.item_serial_number)
-        self.parsed = False
-        self.can_parse = True
+        self.serial.set_serial(new_data)
 
-    @staticmethod
-    def decode_serial_base64(new_data):
+    def get_serial_number(self, orig_seed=False):
         """
-        Overwrites this item with a new one, from a base64 encoding
+        Returns the binary item serial number.  If `orig_seed` is `True`, the
+        serial number will use the same seed that was used in the savegame.
+        Otherwise, it will use a seed of `0`, which will then be unencrypted.
         """
-        if not new_data.lower().startswith('bl3(') or not new_data.endswith(')'):
-            raise Exception('Unknown item format: {}'.format(new_data))
-        encoded = new_data[4:-1]
-        return base64.b64decode(encoded)
+        return self.serial.get_serial_number(orig_seed)
+
+    def get_serial_base64(self, orig_seed=False):
+        """
+        Returns the base64-encoded item serial number.  If `orig_seed` is
+        `True`, the serial number will use the same seed that was used in the
+        savegame.  Otherwise, it will use a seed of `0`, which will then be
+        unencrypted.
+        """
+        return self.serial.get_serial_base64(orig_seed)
+
+    @property
+    def eng_name(self):
+        """
+        Returns the English name of this item, as best we can tell
+        """
+        return self.serial.eng_name
+
+    @property
+    def balance_short(self):
+        """
+        Returns the 'short' balance name for the item, if possible.  This is
+        generally what'd be useful to a user without being a long full-object
+        name.
+        """
+        return self.serial.balance_short
+
+    @property
+    def level(self):
+        """
+        Returns the level of this item
+        """
+        return self.serial.level
+
+    @level.setter
+    def level(self, new_level):
+        """
+        Sets the level of this item
+        """
+        self.serial.level = new_level
+        self.protobuf.item_serial_number = self.serial.serial
 
 class BL3EquipSlot(object):
     """
@@ -814,8 +544,8 @@ class BL3Save(object):
 
     def get_max_playthrough_with_data(self):
         """
-        Returns the maximum playthrough for which we have actual data in
-        the savegame.  Even if TVHM is unlocked, for instance, we may
+        Returns the maximum playthrough (zero-indexed) for which we have actual
+        data in the savegame.  Even if TVHM is unlocked, for instance, we may
         only have NVHM data in the savefile.
         """
         # Really I don't think that any of these numbers will ever be
@@ -825,7 +555,7 @@ class BL3Save(object):
                 len(self.save.active_travel_stations_for_playthrough),
                 len(self.save.last_active_travel_station_for_playthrough),
                 len(self.save.game_state_save_data_for_playthrough),
-                )
+                )-1
 
     def get_pt_mayhem_levels(self):
         """
@@ -880,6 +610,19 @@ class BL3Save(object):
             del self.save.game_state_save_data_for_playthrough[to_pt]
             self.save.game_state_save_data_for_playthrough.insert(to_pt, from_obj.save.game_state_save_data_for_playthrough[from_pt])
 
+    def clear_game_state_pt(self, playthrough):
+        """
+        Clears out all game state data from the given `playthrough` (zero-indexed).  The
+        `playthrough` given must be the *last* one in the list.
+        """
+        if playthrough < 0:
+            raise Exception('playthrough cannot be negative')
+        if (len(self.save.game_state_save_data_for_playthrough)-1) != playthrough:
+            raise Exception('playthrough must be {} for this save'.format(
+                len(self.save.game_state_save_data_for_playthrough)-1,
+                ))
+        self.save.game_state_save_data_for_playthrough.pop()
+
     def get_pt_last_stations(self):
         """
         Returns a list of the object names of the last station (fast travel,
@@ -920,6 +663,19 @@ class BL3Save(object):
         else:
             self.save.last_active_travel_station_for_playthrough[to_pt] = from_obj.save.last_active_travel_station_for_playthrough[from_pt]
 
+    def clear_last_station_pt(self, playthrough):
+        """
+        Clears out all Last Station Visited data from the given `playthrough` (zero-indexed).  The
+        `playthrough` given must be the *last* one in the list.
+        """
+        if playthrough < 0:
+            raise Exception('playthrough cannot be negative')
+        if (len(self.save.last_active_travel_station_for_playthrough)-1) != playthrough:
+            raise Exception('playthrough must be {} for this save'.format(
+                len(self.save.last_active_travel_station_for_playthrough)-1,
+                ))
+        self.save.last_active_travel_station_for_playthrough.pop()
+
     def get_pt_last_maps(self, eng=False):
         """
         Returns a list maps which the player has been in, for each Playthrough.
@@ -1018,17 +774,30 @@ class BL3Save(object):
             del self.save.active_travel_stations_for_playthrough[to_pt]
             self.save.active_travel_stations_for_playthrough.insert(to_pt, from_obj.save.active_travel_stations_for_playthrough[from_pt])
 
-    def get_pt_active_mission_lists(self, eng=False):
+    def clear_active_ft_stations_pt(self, playthrough):
         """
-        Returns a list of active missions for each Playthrough.  Missions will
-        be in their object name by default, or their English names if `eng` is
-        `True`.
+        Clears out all active Fast Travel data from the given `playthrough` (zero-indexed).  The
+        `playthrough` given must be the *last* one in the list.
+        """
+        if playthrough < 0:
+            raise Exception('playthrough cannot be negative')
+        if (len(self.save.active_travel_stations_for_playthrough)-1) != playthrough:
+            raise Exception('playthrough must be {} for this save'.format(
+                len(self.save.active_travel_stations_for_playthrough)-1,
+                ))
+        self.save.active_travel_stations_for_playthrough.pop()
+
+    def get_pt_mission_lists(self, mission_status, eng=False):
+        """
+        Returns a list of missions in the given `mission_status`, for each
+        Playthrough.  Missions will be in their object name by default, or
+        their English names if `eng` is `True`.
         """
         to_ret = []
         for pt in self.save.mission_playthroughs_data:
             active_missions = []
             for mission in pt.mission_list:
-                if mission.status == MissionState.MS_Active:
+                if mission.status == mission_status:
                     mission_name = mission.mission_class_path
                     if eng:
                         if mission_name.lower() in mission_to_name:
@@ -1039,28 +808,56 @@ class BL3Save(object):
             to_ret.append(active_missions)
         return to_ret
 
-    def get_pt_active_mission_list(self, pt, eng=False):
+    def get_pt_active_mission_lists(self, eng=False):
         """
-        Returns a list of active mission object names for the given
+        Returns a list of active missions for each Playthrough.  Missions will
+        be in their object name by default, or their English names if `eng` is
+        `True`.
+        """
+        return self.get_pt_mission_lists(MissionState.MS_Active, eng)
+
+    def get_pt_completed_mission_lists(self, eng=False):
+        """
+        Returns a list of completed missions for each Playthrough.  Missions will
+        be in their object name by default, or their English names if `eng` is
+        `True`.
+        """
+        return self.get_pt_mission_lists(MissionState.MS_Complete, eng)
+
+    def get_pt_mission_list(self, pt, mission_status, eng=False):
+        """
+        Returns a list of missions in the given `mission_status`, for the given
         Playthrough (zero-indexed).  Missions will be in their object name
         by default, or their English names if `eng` is `True`
         """
-        missions = self.get_pt_active_mission_lists(eng=eng)
+        missions = self.get_pt_mission_lists(mission_status, eng=eng)
         if len(missions) > pt:
             return missions[pt]
         return None
 
+    def get_pt_active_mission_list(self, pt, eng=False):
+        """
+        Returns a list of active missions for the given Playthrough (zero-indexed).
+        Missions will be in their object name by default, or their English names
+        if `eng` is `True`
+        """
+        return self.get_pt_mission_list(pt, MissionState.MS_Active, eng)
+
+    def get_pt_completed_mission_list(self, pt, eng=False):
+        """
+        Returns a list of completed missions for the given Playthrough (zero-indexed).
+        Missions will be in their object name by default, or their English names
+        if `eng` is `True`
+        """
+        return self.get_pt_mission_list(pt, MissionState.MS_Complete, eng)
+
     def get_pt_completed_mission_counts(self):
         """
         Returns a count of completed missions for each Playthrough.
         """
         to_ret = []
-        for pt in self.save.mission_playthroughs_data:
-            mission_count = 0
-            for mission in pt.mission_list:
-                if mission.status == MissionState.MS_Complete:
-                    mission_count += 1
-            to_ret.append(mission_count)
+        for missions in self.get_pt_completed_mission_lists():
+            to_ret.append(len(missions))
         return to_ret
 
     def get_pt_completed_mission_count(self, pt):
@@ -1098,6 +895,19 @@ class BL3Save(object):
             del self.save.mission_playthroughs_data[to_pt]
             self.save.mission_playthroughs_data.insert(to_pt, from_obj.save.mission_playthroughs_data[from_pt])
 
+    def clear_mission_pt(self, playthrough):
+        """
+        Clears out all mission data from the given `playthrough` (zero-indexed).  The
+        `playthrough` given must be the *last* one in the list.
+        """
+        if playthrough < 0:
+            raise Exception('playthrough cannot be negative')
+        if (len(self.save.mission_playthroughs_data)-1) != playthrough:
+            raise Exception('playthrough must be {} for this save'.format(
+                len(self.save.mission_playthroughs_data)-1,
+                ))
+        self.save.mission_playthroughs_data.pop()
+
     def copy_playthrough_data(self, from_obj=None, from_pt=0, to_pt=1):
         """
         Copies playthrough-specific data from one playthrough to another (zero-indexed).  Currently
@@ -1114,6 +924,27 @@ class BL3Save(object):
         self.copy_last_station_pt(from_obj=from_obj, from_pt=from_pt, to_pt=to_pt)
         self.copy_game_state_pt(from_obj=from_obj, from_pt=from_pt, to_pt=to_pt)
 
+    def clear_playthrough_data(self, playthrough):
+        """
+        Clears out all data from the given `playthrough` (zero-indexed), including any higher
+        playthroughs.  This is completely untested for clearing NVHM data, but is used by
+        the cli_edit to clear out THVM.  This supports clearing higher levels just because my
+        original scripts to generate my BL3 Save Archive accidentally unlocked an extra
+        playthrough higher than it intended to, so this will allow me to "clear TVHM" while
+        also getting rid of that higher playthrough data.
+
+        It's permissible to pass a `playthrough` higher than the max that we have data for,
+        since THVM can be technically unlocked without actually having THVM data in the save
+        file.
+        """
+        max_pt = self.get_max_playthrough_with_data()
+        if playthrough <= max_pt:
+            for pt in range(max_pt, playthrough-1, -1):
+                self.clear_mission_pt(pt)
+                self.clear_active_ft_stations_pt(pt)
+                self.clear_last_station_pt(pt)
+                self.clear_game_state_pt(pt)
+
     def get_items(self):
         """
         Returns a list of the character's inventory items, as BL3Item objects.
@@ -1226,7 +1057,7 @@ class BL3Save(object):
         Creates a new item from the base64-encoded (and "BL3()"-wrapped)
         `item_serial_b64`, which can later be added to our item list.
         """
-        return self.create_new_item(BL3Item.decode_serial_base64(item_serial_b64))
+        return self.create_new_item(datalib.BL3Serial.decode_serial_base64(item_serial_b64))
 
     def add_new_item(self, item_serial):
         """
@@ -1243,7 +1074,7 @@ class BL3Save(object):
         "BL3()"-wrapped) `item_serial_b64`.  Returns a tuple containing the
         new BL3Item object itself, and its new index in our item list.
         """
-        return self.add_new_item(BL3Item.decode_serial_base64(item_serial_b64))
+        return self.add_new_item(datalib.BL3Serial.decode_serial_base64(item_serial_b64))
 
     def overwrite_item_in_slot(self, slot, itemdata):
         """
@@ -1404,6 +1235,12 @@ class BL3Save(object):
                 ammo_key = ammoobj_to_ammo[pool.resource_path]
                 pool.amount = ammo_to_max[ammo_key]
 
+    def get_all_challenges_raw(self):
+        """
+        Returns the savegame's list of all challenges, as the actual protobuf objects.
+        """
+        return sorted(self.save.challenge_data, key=lambda chal: chal.challenge_class_path)
+
     def get_interesting_challenges(self, eng=False):
         """
         Returns a dict containing the challenge type and completed status.  The challenge
@@ -1481,13 +1318,18 @@ class BL3Save(object):
         """
         to_ret = {}
         for v in self.save.vehicles_unlocked_data:
-            key = chassis_to_vehicle[v.asset_path]
-            if eng:
-                key = vehicle_to_eng[key]
-            if key in to_ret:
-                to_ret[key] += 1
-            else:
-                to_ret[key] = 1
+            # Some DLC3 "vehicles" are already showing up in saves, at least this one:
+            # /Geranium/Vehicles/Horse/Design/WT_Horse_Biobeast.WT_Horse_Biobeast
+            # So check for that and don't try to load if we don't know about the
+            # vehicle type.
+            if v.asset_path in chassis_to_vehicle:
+                key = chassis_to_vehicle[v.asset_path]
+                if eng:
+                    key = vehicle_to_eng[key]
+                if key in to_ret:
+                    to_ret[key] += 1
+                else:
+                    to_ret[key] = 1
         return to_ret
 
     def get_vehicle_chassis_count(self, vehicle_type):
