@@ -60,9 +60,9 @@ def main():
             epilog="""
                 The default output type of "savegame" will output theoretically-valid
                 savegames which can be loaded into BL3.  The output type "protobuf"
-                will save out the extracted, decrypted protobufs.  These protobufs
-                CANNOT be read back into the file using this editor, so that's a
-                write-only operation.  The output type "items" will output a text
+                will save out the extracted, decrypted protobufs.  The output
+                type "json" will output a JSON-encoded version of the protobufs
+                in question.  The output type "items" will output a text
                 file containing base64-encoded representations of the user's
                 inventory.  These can be read back in using the -i/--import-items
                 option.  Note that these are NOT the same as the item strings used
@@ -129,7 +129,7 @@ def main():
 
     parser.add_argument('--mayhem',
             type=int,
-            choices=range(5),
+            choices=range(11),
             help='Set the mayhem mode for all playthroughs (mostly useful for Normal mode)',
             )
 
@@ -157,12 +157,20 @@ def main():
             help='Game features to unlock',
             )
 
-    parser.add_argument('--copy-nvhm',
+    tvhmgroup = parser.add_mutually_exclusive_group()
+
+    tvhmgroup.add_argument('--copy-nvhm',
             dest='copy_nvhm',
             action='store_true',
             help='Copies NVHM/Normal state to TVHM',
             )
 
+    tvhmgroup.add_argument('--unfinish-nvhm',
+            dest='unfinish_nvhm',
+            action='store_true',
+            help='"Un-finishes" the game: remove all TVHM data and set Playthrough 1 to Not Completed',
+            )
+
     parser.add_argument('-i', '--import-items',
             dest='import_items',
             type=str,
@@ -172,7 +180,8 @@ def main():
     parser.add_argument('--allow-fabricator',
             dest='allow_fabricator',
             action='store_true',
-            help='Allow importing Fabricator when importing items from file')
+            help='Allow importing Fabricator when importing items from file',
+            )
 
     # Positional args
     parser.add_argument('input_filename',
@@ -196,6 +205,10 @@ def main():
         args.unlock['artifactslot'] = True
         args.unlock['comslot'] = True
 
+    # Make sure we're not trying to clear and unlock THVM at the same time
+    if 'tvhm' in args.unlock and args.unfinish_nvhm:
+        raise argparse.ArgumentTypeError('Cannot both unlock TVHM and un-finish NVHM')
+
     # Set max level arg
     if args.level_max:
         args.level = bl3save.max_level
@@ -245,6 +258,7 @@ def main():
         args.import_items,
         args.items_to_char,
         args.item_levels,
+        args.unfinish_nvhm,
         ])
 
     # Make changes
@@ -438,6 +452,12 @@ def main():
             if not args.quiet:
                 print(' - Copying NVHM state to TVHM')
             save.copy_playthrough_data()
+        elif args.unfinish_nvhm:
+            if not args.quiet:
+                print(' - Un-finishing NVHM state entirely')
+            # ... or clearing TVHM state entirely.
+            save.set_playthroughs_completed(0)
+            save.clear_playthrough_data(1)
 
         # Newline at the end of all this.
         if not args.quiet:
