@@ -43,6 +43,26 @@ def main():
             help='Show all available information',
             )
 
+    parser.add_argument('-i', '--items',
+            action='store_true',
+            help='Show inventory items',
+            )
+
+    parser.add_argument('--all-missions',
+            dest='all_missions',
+            action='store_true',
+            help='Show all missions')
+
+    parser.add_argument('--all-challenges',
+            dest='all_challenges',
+            action='store_true',
+            help='Show all challenges')
+
+    parser.add_argument('--fast-travel',
+            dest='fast_travel',
+            action='store_true',
+            help='Show all unlocked Fast Travel stations')
+
     parser.add_argument('filename',
             help='Filename to process',
             )
@@ -79,12 +99,12 @@ def main():
     print('Playthroughs Completed: {}'.format(save.get_playthroughs_completed()))
 
     # Playthrough-specific Data
-    for pt, (mayhem, mapname, stations, missions, mission_count) in enumerate(itertools.zip_longest(
+    for pt, (mayhem, mapname, stations, active_missions, completed_missions) in enumerate(itertools.zip_longest(
             save.get_pt_mayhem_levels(),
             save.get_pt_last_maps(True),
             save.get_pt_active_ft_station_lists(),
             save.get_pt_active_mission_lists(True),
-            save.get_pt_completed_mission_counts(),
+            save.get_pt_completed_mission_lists(True),
             )):
 
         print('Playthrough {} Info:'.format(pt+1))
@@ -98,27 +118,47 @@ def main():
             print(' - In Map: {}'.format(mapname))
 
         # FT Stations
-        if args.verbose:
+        if args.verbose or args.fast_travel:
             if stations is not None:
                 if len(stations) == 0:
-                    print(' - No Active FT Stations')
+                    print(' - No Active Fast Travel Stations')
                 else:
-                    print(' - Active FT Stations:'.format(pt+1))
+                    print(' - Active Fast Travel Stations:'.format(pt+1))
                     for station in stations:
                         print('   - {}'.format(station))
 
         # Missions
-        if missions is not None:
-            if len(missions) == 0:
+        if active_missions is not None:
+            if len(active_missions) == 0:
                 print(' - No Active Missions')
             else:
                 print(' - Active Missions:')
-                for mission in sorted(missions):
+                for mission in sorted(active_missions):
                     print('   - {}'.format(mission))
 
         # Completed mission count
-        if mission_count is not None:
-            print(' - Missions completed: {}'.format(mission_count))
+        if completed_missions is not None:
+            print(' - Missions completed: {}'.format(len(completed_missions)))
+
+            # Show all missions if need be
+            if args.verbose or args.all_missions:
+                for mission in sorted(completed_missions):
+                    print('   - {}'.format(mission))
+
+            # "Important" missions - I'm torn as to whether or not this kind of thing
+            # should be in bl3save.py itself, or at least some constants in __init__.py
+            mission_set = set(completed_missions)
+            importants = []
+            if 'Divine Retribution' in mission_set:
+                importants.append('Main Game')
+            if 'All Bets Off' in mission_set:
+                importants.append('DLC1 - Moxxi\'s Heist of the Handsome Jackpot')
+            if 'The Call of Gythian' in mission_set:
+                importants.append('DLC2 - Guns, Love, and Tentacles')
+            if len(importants) > 0:
+                print(' - Mission Milestones:')
+                for important in importants:
+                    print('   - Finished: {}'.format(important))
 
     # Inventory Slots that we care about
     print('Unlockable Inventory Slots:')
@@ -129,7 +169,7 @@ def main():
             ))
 
     # Inventory
-    if args.verbose:
+    if args.verbose or args.items:
         items = save.get_items()
         if len(items) == 0:
             print('Nothing in Inventory')
@@ -145,7 +185,7 @@ def main():
                 print(line)
 
     # Equipped Items
-    if args.verbose:
+    if args.verbose or args.items:
         items = save.get_equipped_items(True)
         if any(items.values()):
             print('Equipped Items:')
@@ -180,6 +220,17 @@ def main():
     for challenge, status in save.get_interesting_challenges(True).items():
         print(' - {}: {}'.format(challenge, status))
 
+    # "raw" Challenges
+    if args.verbose or args.all_challenges:
+        print('All Challenges:')
+        for challenge in save.get_all_challenges_raw():
+            print(' - {} (Completed: {}, Counter: {}, Progress: {})'.format(
+                challenge.challenge_class_path,
+                challenge.currently_completed,
+                challenge.progress_counter,
+                challenge.completed_progress_level,
+                ))
+
     # Vehicle unlocks
     print('Unlocked Vehicle Parts:')
     for vehicle, chassis_count in save.get_vehicle_chassis_counts().items():
